package wspack;

import java.net.*;
import java.io.*;
import javax.servlet.http.*;
import org.w3c.dom.*;
import toolspack.*;
import toolspack.udppack.*;


/**
 * <p>Copyright: Copyright (c) 2002</p>
 * @author Stefano Ricciarelli
 * @version 1.5 - Note di versione: Si tratta dell'infrastruttura sviluppata come progetto di IS
 * resa adatta per l'esecuzione distribuita. Funziona ancora esattamente come prima, non vi 
 * un NMS, non vi sono limitazioni sugli eventi e nessun'altra innovazione o diversit.
 * Semplicemente a livello implementativo vi sono programmi (vedi anche le due opzioni del run)
 * che comunicano attraverso socket. Prima fare "Esegui RN" poi "Esegui WS"
 * DATA: 16-08-2002
 * ORA:  19:30
 * NOTE: tutto OK!
 */

public class RequestExecutor {

  private WSConfigurator config = WSConfigurator.getInstance();
  private short getRnServiceId      = 11;
  private short removeRnServiceId   = 20;


  public RequestExecutor() {
  }

  public synchronized String execute(HttpServletRequest request){
    HttpSession session        = request.getSession();
    String      sessionID      = session.getId();
    XmlTools    xmlTools       = XmlTools.getInstance();
    String      responseString;
    Address     remoteAddress;
    Document    requestXML;
    Socket      socket = null;
    try {
      //creo la requestXML
      requestXML = RequestXMLBuilder.buildRequestXML(request);
      //Logger.print(this, "creata la requestXML");
      int aid = 0;
      if (session.isNew()) {
        Logger.print(this, "Nuova sessione");  //NUOVA SESSIONE
        //PROTOCOLLO RETI numero ** 6 ** - RICHIESTA RN PRESSO NMS - PASSO WSID, ATTENDO RNADDRESSE E AID
        try {
          Logger.print(this, "Richiedo al NMS un RN su cui eseguire la nuova sessione....");
          byte[] response = mySendAndReceive( ByteUtil.toByteArray(config.getWsid()),  getRnServiceId);
          if (response == null) {
            Logger.print(this, "ERRORE: impossibile ottenere da NMS un nodo su cui eseguire la nuova sessione ");
            return null;
          }
          DataInputStream din = new DataInputStream( new ByteArrayInputStream(response) );
          short iplength  = din.readShort();
          if (iplength == -1) {
            Logger.print(this, "Non mi  stato restituito nessun RN");
            return null;
          }
          byte[] ipraw    = new byte[iplength];
                            din.read(ipraw, 0, iplength);
          int port        = din.readInt();
          aid             = din.readInt();
          remoteAddress   = new Address(ipraw, port);
        } catch (Exception ex) {
          System.out.println("ERRORE " + ex);
          return null;
        }
        Logger.print(this, "Ho ottenuto dal NMS un RN su cui eseguire la nuova sessione");
        session.setAttribute("aid", new Integer(aid));
        session.setAttribute("remoteAddress", remoteAddress);
      }
      else { //la sessione  gi esistente
        Logger.print(this, "Sessione esistente");
        aid           = ((Integer)session.getAttribute("aid")).intValue();
        remoteAddress =  (Address)session.getAttribute("remoteAddress");
      }
      requestXML.getDocumentElement().setAttribute("wsid", String.valueOf(config.getWsid()) );
      requestXML.getDocumentElement().setAttribute("aid" , String.valueOf(aid) ); //non usato
      Logger.appendErrorXML(requestXML, "requesXML");

      //********************** Apro la socket col RN ****************************//
      try {
        socket = new Socket(remoteAddress.getRemoteHost(), remoteAddress.getRemotePort());
        socket.setSoTimeout(config.getTcpTimeout());
        Logger.print(this, "Aperta socket con il RN " + remoteAddress + ". Timeout = " + config.getTcpTimeout());
        //Spedisco la richiesta
        ObjectOutputStream out = new ObjectOutputStream(socket.getOutputStream());
        out.writeObject(xmlTools.XML2string(requestXML));
        //Logger.print(this, "Richiesta spedita al RN, in attesa di risposta... ");
      } catch(Exception ex2) {
        Logger.print(this, "ERRORE cercando di connettersi al RN " + remoteAddress + ". Spedisco al NMS richiesta di rimozione per questo RN");
        remove(remoteAddress);
      }
      //ricevo la risposta
      //ATTENZIONE: la seguente operazione di creazione dell' ObjectInputStream  bloccante!!
      //Se dall'altro lato non  stata fatta una writeObject mi fermo qui e non quando
      //faccio la readObject!! Forse ci  dovuto a una qualche bufferizzazione
      ObjectInputStream in = new ObjectInputStream(socket.getInputStream());
      responseString = (String)in.readObject();
      Logger.print(this, "Risposta del RN giunta correttamente");
      socket.close();
      return responseString;
    } catch(Exception ex) {
      //azioni da intraprendere se per esempio l'executor node non possiede pi lui i bean
      Logger.print(this, "Errore nell'esecuzione remota " + ex);
      try{socket.close();}catch (Exception e2){}
      return null;
    }
  }//execute


  private byte[] mySendAndReceive (byte[] request, short serviceId) {
    try {
      IUDPServiceSocket udpSocket = config.getUdpSocket();
      //la seguente non genera errori, non tenta un effettivo collegamento
      IServiceEndPoint sep = udpSocket.getNewServiceEndPoint(config.getNMSAddress1(), config.getNMSPort1(), serviceId);
      int i=0;
      int timeout = config.getUdpBaseTimeout();
      byte[] response = null;
      int j = 0;
      while (j < 2) {
        while ( (response == null) && i<config.getUdpRetry() ) {
          Logger.print(this, "Protocollo del servizio " + serviceId + ": tentativo " + (i+1) + " con timeout " + timeout);
          try {
            response = sep.sendAndReceive(request, timeout);
          } catch (Exception ex2) {
            Logger.print(this, "ATTENZIONE: ricevuta un'eccezione da sendAndReceive " + ex2);
            response = null;
          }
          timeout = (int) (timeout * config.getUdpMulTimeout());
          i++;
        }//while interno
        if ( (response == null) && (j == 0) ) {
          Logger.print(this, "Tentativi falliti, provo a collegarmi all'altro NMS ");
          config.swapNMS(); //inverto i NMS perch anche tutte le altre operazioni accedano direttamente al NMS funzionante
          sep = udpSocket.getNewServiceEndPoint(config.getNMSAddress1(), config.getNMSPort1(), serviceId);
          j++;
          i=0;
          timeout = config.getUdpBaseTimeout();
        } else j = 2;
      }//while esterno
      //se ho ottenuto comunque un errore probabilmente non era per colpa del NMS: rimetto a posto gli indirizzi
      if (response == null) config.swapNMS();
      return response;
    } catch (Exception ex) {
      Logger.print(this, "ERRORE: nel Protocollo del servizio " + serviceId + " " + ex);
      return null;
    }//catch
  }//mySendAndReceive


  private void remove(Address addr) {
    //PROTOCOLLO RETI numero ** 7 ** - RICHIESTA RIMOZIONE RN PRESSO NMS - PASSO  RNADDRESS
    try {
      IUDPServiceSocket udpSocket = config.getUdpSocket();
      //la seguente non genera errori, non tenta un effettivo collegamento
      IServiceEndPoint sep = udpSocket.getNewServiceEndPoint(config.getNMSAddress1(), config.getNMSPort1(), removeRnServiceId);
      boolean b = sep.send(addr.getSerializedForm());
      if (!b) {
        Logger.print(this, "Send fallita, provo a collegarmi all'altro NMS ");
        config.swapNMS(); //inverto i NMS perch anche tutte le altre operazioni accedano direttamente al NMS funzionante
        sep = udpSocket.getNewServiceEndPoint(config.getNMSAddress1(), config.getNMSPort1(), removeRnServiceId);
        sep.send(addr.getSerializedForm());
      }
    } catch (Exception ex) {
      Logger.print(this, "ERRORE spedendo richiesta di rimozione RN " + ex);
    }
  }


}//fine classe

