package wspack;
//NUOVA CLASSE PROGETTO RETI

import java.net.URI;
import java.util.StringTokenizer;
import javax.servlet.http.*;
import org.w3c.dom.*;
import toolspack.*;

/**
 * <p>Copyright: Copyright (c) 2002</p>
 * @author Stefano Ricciarelli
 * @version 1.5 - Note di versione: Si tratta dell'infrastruttura sviluppata come progetto di IS
 * resa adatta per l'esecuzione distribuita. Funziona ancora esattamente come prima, non vi 
 * un NMS, non vi sono limitazioni sugli eventi e nessun'altra innovazione o diversit.
 * Semplicemente a livello implementativo vi sono programmi (vedi anche le due opzioni del run)
 * che comunicano attraverso socket. Prima fare "Esegui RN" poi "Esegui WS"
 * DATA: 16-08-2002
 * ORA:  19:30
 * NOTE: tutto OK!
 */

public class RequestXMLBuilder {

  private static XmlTools xmlTools;

  public RequestXMLBuilder() {
  }

  /**
   * Costruisce requestXML a partire da httpRequest. Il compito coinvolge alcune problematiche.
   * Anzitutto l'Enumeration names = httpRequest.getParameterNames(), non garantisce l'ordine
   * dei parametri cos come sono scritti nell'URL. Per questo motivo si procede ad un parsign
   * della richiesta query = httpRequest.getQueryString() attraverso lo StringTokenizer.
   * Secondariamente vi  il problema della codifica dei caratteri dell'URL identificati da
   * %xx con xx=codifica esadecimale USASCII. Per questo motivo si crea un URI(since jdk1.4)
   * al quale chiedere la queryString che verr restituita con i caratteri gi decodificati
   * @param httpRequest la richiesta arrivata alla servlet
   * @return la richiesta i formato XML
   */
  public static synchronized Document buildRequestXML(HttpServletRequest httpRequest) {
    //creo la nuova requestXML a partire dalla richiesta HTTP
    xmlTools = XmlTools.getInstance();
    Document requestXML = xmlTools.newDocument();
    Element request = requestXML.createElement("request");
    requestXML.appendChild(request);
    //estraggo la queryString (decodificata) dalla richiesta
    String query = httpRequest.getQueryString();
    StringBuffer bufferURL = httpRequest.getRequestURL();
    String requestURI;
    if (query==null) query="";
    if (query.equals("")) requestURI = bufferURL.toString();
    else requestURI = bufferURL.append('?').append(query).toString();
    try {
      URI uri  = new URI(requestURI); //since jdk1.4
      String decodedQuery = uri.getQuery();
      //parserizzo ogni coppia nome=valore
      StringTokenizer st = new StringTokenizer(query, new String("&"));
      //se non c' alcun parametro restituisco un requestXML con le sole requestProperties
      if (!st.hasMoreTokens()) {
        requestXML = appendRequestProperties(httpRequest, requestXML);
        return requestXML;
      }
      //per ogni coppia nome=valore parserizzo il nome e il valore
      int j=0;
      String  paramName, paramValue;
      StringTokenizer st2;
      boolean start = true;
      //Variabile indicante l'attuale nodo action a cui aggiungere i parametri
      Element actualActionElement = null;
      while (st.hasMoreTokens()) {
        st2          = new StringTokenizer(st.nextToken(), new String("="));
        paramName    = st2.nextToken();
        paramValue   = st2.nextToken();
        if (st2.hasMoreTokens()) {Logger.print("errore parserizzando "+st2+" in "+st); return null;}
        Logger.print("RequestXMLBuilder: parametro "+j+" nome: "+paramName+" valore: "+paramValue);
        // se si tratta del primo parametro deve necessariamente essere un "action"
        if (start) {
         if (!paramName.equals("action"))
          {Logger.print("ERRORE: url non corretto"); return requestXML;} //errore
         start = false;
        }
        if (paramName.equals("action")) {
          Element action = requestXML.createElement("action");
          action.setAttribute("name", paramValue);
          request.appendChild(action);
          actualActionElement = action;
        }
        else{ //parametri della action
          Element param = requestXML.createElement("param");
          param.setAttribute(paramName, paramValue);
          actualActionElement.appendChild(param);
        }
        j++;
      }//while
    } catch (Exception ex) {Logger.print("ERRORE costruendo l'URI " + ex);}
    requestXML = appendRequestProperties(httpRequest, requestXML);
    return requestXML;
  }

  /**
   * Aggiunge al documento requestXML un tag <requestProperties> contenenete vari attributi
   * che indicano propriet della richiesta httpRequest
   * @param httpRequest la richiesta http ricevuta dalla servlet
   * @param requestXML la richiesta XML
   * @return la nuova richiesta XML contenente anche le propriet della richiesta
   */
  private static Document appendRequestProperties(HttpServletRequest httpRequest, Document requestXML) {
    //Inserisco il nodo requestProperties
    Element rp = requestXML.createElement("requestProperties");
    Element e; String s;
    String query = httpRequest.getQueryString();
    StringBuffer bufferURL = httpRequest.getRequestURL();
    String requestURI;
    if (query==null) query="";
    if (query.equals("")) requestURI = bufferURL.toString();
    else requestURI = bufferURL.append('?').append(query).toString();
    rp.setAttribute("requestURL", (requestURI==null)? "" : requestURI);
    s = httpRequest.getContextPath();
    rp.setAttribute("contextPath", (s==null)? "" : s);
    s = httpRequest.getPathInfo();
    rp.setAttribute("pathInfo", (s==null)? "" : s);
    s = httpRequest.getServletPath();
    rp.setAttribute("servletPath", (s==null)? "" : s);
    s = httpRequest.getQueryString();
    rp.setAttribute("queryString", (s==null)? "" : s);
    s = httpRequest.getRemoteAddr();
    rp.setAttribute("remoteAddr", (s==null)? "" : s);
    s = httpRequest.getRemoteHost();
    rp.setAttribute("remoteHost", (s==null)? "" : s);
    s = httpRequest.getRemoteUser();
    rp.setAttribute("remoteUser", (s==null)? "" : s);
    s = httpRequest.getProtocol();
    rp.setAttribute("protocol", (s==null)? "" : s);
    s = httpRequest.getServerName();
    rp.setAttribute("serverName", (s==null)? "" : s);
    s = Integer.toString(httpRequest.getServerPort());
    rp.setAttribute("serverPort", (s==null)? "" : s);

    HttpSession session = httpRequest.getSession();
    s = session.getId();
    rp.setAttribute("sessionID", (s==null)? "" : s);
    s = Boolean.toString(session.isNew());
    rp.setAttribute("sessionIsNew", (s==null)? "" : s);
    s = Long.toString(session.getCreationTime());
    rp.setAttribute("sessionCreationTime", (s==null)? "" : s);
    s = Long.toString(session.getLastAccessedTime());
    rp.setAttribute("sessionLastAccessedTime", (s==null)? "" : s);

    requestXML.getDocumentElement().appendChild(rp);
    return requestXML;
  }




}