package wspack;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import org.w3c.dom.Document;
import toolspack.*;
import java.net.InetAddress;
import toolspack.udppack.*;

/**
 * <p>Copyright: Copyright (c) 2002</p>
 * @author Stefano Ricciarelli
 * @version 1.5 - Note di versione: Si tratta dell'infrastruttura sviluppata come progetto di IS
 * resa adatta per l'esecuzione distribuita. Funziona ancora esattamente come prima, non vi 
 * un NMS, non vi sono limitazioni sugli eventi e nessun'altra innovazione o diversit.
 * Semplicemente a livello implementativo vi sono programmi (vedi anche le due opzioni del run)
 * che comunicano attraverso socket. Prima fare "Esegui RN" poi "Esegui WS"
 * DATA: 16-08-2002
 * ORA:  19:30
 * NOTE: tutto OK!
 * Servlet che gestisce l'interazione della applicazione con gli utenti mediante il protocollo
 * http. Questa classe  la prima ad essere inizializzata e provvede ad inizializzare tutte
 * le altre classi dell'applicazione
 */

public class ServletRequestManager extends HttpServlet {
  private static final String CONTENT_TYPE = "text/html";
  private static final String DOC_TYPE = "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"\n" +
      "  \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">";
  private static final String ERROR_PAGE = "<p> ERRORE </p>";

  private static boolean init_ok = true;
  private static int sessionTime = 5; // secondi di durata di una sessione
  private static String nmsHostName1 = "169.254.0.1";
  private static int nmsUdpPort1     = 4000;
  private static String nmsHostName2 = "169.254.0.2";
  private static int nmsUdpPort2     = 4000;
  private static int myUdpPort = 3000;
  private static short fixedMsgLength = 1024;
  private static short wsRegisterServiceId = 10;
  private static int udpServicesBaseTimeout        = 500;
  private static float timeoutMulFactor = 1.5F; //ricorda che per java il default per i nuemri in virgola mobile  double
  private static int retry              = 3;
  private static int wsid = 999;
  private static int tcpTimeout = 3500;  //tempo che attendo per avere risposta dal RN


  /**
   * Invocato all'avvio della webapplication. Legge gli initParameter della servlet
   * presenti nel file web.xml e provvede ad inizializzare tutti le classi dell'
   * applicaizione
   * @throws ServletException
   */
  public void init() throws ServletException {
    init_ok = true;
    boolean ok = true;
    //String logPath, debug, , printDebugOnConsoleOrFile;
    //String poolInstanceNumber, configPath, pagesPath, sessionTime, debugXML;
    //configPath = this.getInitParameter("configurationBasePath");
    //if (configPath==null) configPath="";
    //logPath = this.getInitParameter("logBasePath");
    //if (logPath==null) logPath="";
    //pagesPath = this.getInitParameter("outputPagesBasePath");
    //if (pagesPath==null) pagesPath="";
    //debug = this.getInitParameter("debug");
    //if (debug==null) debug="";
    //debugXML = this.getInitParameter("debugXML");
    //if (debugXML==null) debugXML="";
    //printDebugOnConsoleOrFile = this.getInitParameter("printDebugOnConsoleOrFile");
    //if (printDebugOnConsoleOrFile==null) printDebugOnConsoleOrFile="";
    //poolInstanceNumber = this.getInitParameter("poolInstanceNumber");
    //sessionTime = this.getInitParameter("sessionTime");
    //Invoco il costruttore di Logger per inizializzare correttamente la classe
    //new Logger(logPath);
    //if (debug.equals("true")) Logger.setDebug(true);
    //else Logger.setDebug(false);
    //if (printDebugOnConsoleOrFile.equals("console")) Logger.setOutputOnConsole(true);
    //else Logger.setOutputOnConsole(false);
    //if (debugXML.equals("true"))  Logger.setDebugXML(true);
    //else Logger.setDebugXML(false);

    //Inizializzo xmlTools
    XmlTools.getInstance();

    //Inizializzo il Configurator del NMS
    WSConfigurator config = WSConfigurator.getInstance();

    try {
      config.setNMSAddress1(InetAddress.getByName(nmsHostName1));
      config.setNMSAddress2(InetAddress.getByName(nmsHostName2));
      config.setMyAddress( InetAddress.getLocalHost());
    } catch (Exception ex) {
      Logger.print(this, "ERRORE nel nome host di NMS");
      ok = false;
      return;
    }
    config.setNMSPort1(       nmsUdpPort1);
    config.setNMSPort2(       nmsUdpPort2);
    config.setMyPort(         myUdpPort);
    config.setUdpBaseTimeout( udpServicesBaseTimeout);
    config.setUdpMulTimeout(  timeoutMulFactor);
    config.setUdpRetry(       retry);
    config.setTcpTimeout(     tcpTimeout);

    config.setUdpSocket( new UDPServiceSocket(config.getMyAddress(), config.getMyPort(), fixedMsgLength) );

    //PROTOCOLLO RETI numero ** 1 ** - REGISTRAZIONE PRESSO NMS - PASSO WSID, ATTENDO OK
    try {
      byte[] response = mySendAndReceive( ByteUtil.toByteArray(wsid),  wsRegisterServiceId, config);
      if (response == null) {
        Logger.print(this, "ERRORE: impossibile registrare questo wsid presso NMS");
        ok = false;
      }else {
        //il wsid  comunque stabilito da WS, NON  ottenuto da NMS. NMS si limita ad accettarlo e a registrarlo
        config.setWsid(wsid);
        String responseStr = (new DataInputStream (new ByteArrayInputStream(response))).readUTF();
        if ( ! responseStr.equals("ok")) {
          Logger.print(this, "ERRORE: la risposta riceuta non  corretta");
          ok = false;
        }
      }//else
    } catch (Exception ex) {
      Logger.print(this, "ERRORE: registrando questo wsid presso NMS");
      ok = false;
    }//catch
    if (ok) Logger.print(this, "INIZIALIZZAZIONE SERVLET OK");
    else { Logger.print(this, "ERRORE INIZIALIZZANDO LA SERVLET"); init_ok = false; }
  }//fine init()

  private byte[] mySendAndReceive (byte[] request, short serviceId, WSConfigurator config) {
    try {
      IUDPServiceSocket udpSocket = config.getUdpSocket();
      //la seguente non genera errori, non tenta un effettivo collegamento
      IServiceEndPoint sep = udpSocket.getNewServiceEndPoint(config.getNMSAddress1(), config.getNMSPort1(), serviceId);
      int i=0;
      int timeout = config.getUdpBaseTimeout();
      byte[] response = null;
      int j = 0;
      while (j < 2) {
        while ( (response == null) && i<config.getUdpRetry() ) {
          Logger.print(this, "Protocollo del servizio " + serviceId + ": tentativo " + (i+1) + " con timeout " + timeout);
          try {
            response = sep.sendAndReceive(request, timeout);
          } catch (Exception ex2) {
            Logger.print(this, "ATTENZIONE: ricevuta un'eccezione da sendAndReceive " + ex2);
            response = null;
          }
          timeout = (int) (timeout * config.getUdpMulTimeout());
          i++;
        }//while interno
        if ( (response == null) && (j == 0) ) {
          Logger.print(this, "Tentativi falliti, provo a collegarmi all'altro NMS ");
          config.swapNMS(); //inverto i NMS perch anche tutte le altre operazioni accedano direttamente al NMS funzionante
          sep = udpSocket.getNewServiceEndPoint(config.getNMSAddress1(), config.getNMSPort1(), serviceId);
          j++;
          i=0;
        } else j = 2;
        timeout = config.getUdpBaseTimeout();
      }//while esterno
      //se ho ottenuto comunque un errore probabilmente non era per colpa del NMS: rimetto a posto gli indirizzi
      if (response == null) config.swapNMS();
      return response;
    } catch (Exception ex) {
      Logger.print(this, "ERRORE: nel Protocollo del servizio " + serviceId + " " + ex);
      return null;
    }//catch
  }//mySendAndReceive

  /************************** DO GET ************************
   * Metodo richiamato ad ogni richiesta http di un utente. Le variabili locali di questo
   * metodo sono copie per ogni richiesta quindi non si hanno problemi legati alla concorrenza
   * @param request la richieta http
   * @param response la risposta prodotta dall'applicazione a seguito della richiesta
   * @throws ServletException
   * @throws IOException
   */
  public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
    HttpSession session = request.getSession();
    session.setMaxInactiveInterval(sessionTime);
    PrintWriter out = response.getWriter();
    if (!init_ok) {
      response.setContentType(CONTENT_TYPE);
      out.println(ERROR_PAGE);
      return;
    }
    Logger.print(this, "******************************* NUOVA RICHIESTA ************************");
    try{
      RequestExecutor exec = new RequestExecutor();
      String response2 = exec.execute(request);
      if (response2 == null) {
        //CASO DI ERRORE
        Logger.print(this, "ERRORE!!");
        response.setContentType(CONTENT_TYPE);
        out.println(ERROR_PAGE);
        try {session.invalidate();} catch (Exception ex2) {}
      }
      else out.println(response2);
    } catch(Exception ex) {
      //CASO DI ERRORE
      Logger.print(this, "ERRORE!!");
      response.setContentType(CONTENT_TYPE);
      out.println(ERROR_PAGE);
      try {session.invalidate();} catch (Exception ex2) {}
    }

  }//doGet

  //Process the HTTP Post request
  public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
    doGet(request, response);
  }
  //Clean up resources
  public void destroy() {
  }

  public void errore() {

  }

}//fine classe




